/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __SPLINE_H
#define __SPLINE_H

#include "point.h"
#include <vector>

using std::vector;

class Spline {
private:
	FPoint start, end;
	FPoint waypoint1, waypoint2, waypoint3;
	float divpos;
	float len1, len2, len3, len4;
	float totlen;

	static float getLength(const FPoint &p1, const FPoint &p2, const FPoint &p3, float *lp1, float *lp2);

public:
	Spline(const FPoint &p1, const FPoint &relpoint1, const FPoint &p2, const FPoint &relpoint2);
	~Spline();
	float getLength();
	void calc(float pos, FPoint *out);
};

class CompSpline {
private:
	vector<Spline*> splines;
	vector<float> lengths;
	FPoint lastPos, lastRel;
	bool inited;
	float totLen;
public:
	CompSpline();
	~CompSpline();
	void pushPoint(float x, float y, float relx, float rely);
	void calc(float pos, FPoint *out);
	float getLength();
};

void initSplines(int tileWidth, int tileHeight);
CompSpline *getSpline(int tile, int rotation, int nr, bool opposite);

#endif
